#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Alphabetize the CMake lists of headers and sources.

Typical usage:
  Tools/alphabetize-cmakelists.py SurgSim/CMakeLists.py
or:
  find . -name 'build*' -prune -o -name CMakeLists.txt -print \
    | xargs python Tools/alphabetize-cmakelists.py
"""

import argparse
import sys
import re

def slurp_lines(file):
  try:
    with open(file, 'r') as f:
      return f.read().splitlines()
  except IOError as e:
    print >> sys.stderr, e
    return None

def spew_lines(file, lines):
  try:
    with open(file, 'wb') as f:
      for line in lines:
        f.write(line + "\n")
      return True
  except IOError as e:
    print >> sys.stderr, e
    return False

def filename_sort_filter(file):
#  return re.sub(r'Tests?$', '', re.sub(r'\.(?:h|cpp)$', '', file)).lower()
# Taking out the string 'Tests' screws up the file ordering test
  return re.sub(r'\.(?:h|cpp)$', '', file).lower()

def alphabetize(file, lines):
  if lines is None:
    return None
  (result, indent, var_name, content) = ([], '', None, None)
  for line in lines:
    if content is not None:
      (xline, had_closing_paren) = re.subn(r'\)\s*$', '', line)
      if re.search(r'[(){}]', xline):
        print >> sys.stderr, file + ": Bad line '" + line + "'"
        # put the content lines back unmodified
        result.extend(content)
        result.append(line)
        continue
      content.append(xline)
      if had_closing_paren:
        content = content[1:]  # strip the original set(... line
        content = filter(lambda x: len(x), map(lambda x: x.strip(), content))
        #print var_name + ":", content
        content = sorted(content, key=filename_sort_filter)
        result.append(indent + "set(" + var_name)
        result.extend(map(lambda x: indent + "\t" + x, content))
        result.append(indent + ")")
        content = None
    else:
      match = re.search(
        r'^(\s*)[Ss][Ee][Tt]\s*\(\s*(\w+(?:HEADERS|SOURCES))\s*$', line)
      if match:
        indent = match.group(1)
        var_name = match.group(2)
        content = [ line ]  # include original line, will be removed later
      else:
        result.append(line)
  if result == lines:
    return None
  return result

if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description='Alphabetize CMake lists of headers and sources.')
  parser.add_argument('files', metavar='FILE', nargs='*',
                      help='The file names to reformat.')
  args = parser.parse_args()

  touched = False
  for file in args.files:
    if not re.search(r'\.(?:txt|cmake)$', file):
      print >> sys.stderr, file + ": not a CMake input file!"
    else:
      lines = alphabetize(file, slurp_lines(file))
      if lines is not None:
        spew_lines(file, lines)
        print "Updated", file
        touched = True

  if not touched:
    print "{}: Nothing to update!".format(sys.argv[0])



#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Remove all trailing whitespace from a file.

Typical usage:
  Tools/remove-trailing-whitespace.py Foo.h
or:
  find SurgSim \( -name '*.h' -o -name '*.cpp' \) -print \
    | xargs python Tools/remove-trailing-whitespace.py
"""

import argparse
import sys
import re

def slurp_raw_lines(file):
  try:
    with open(file, 'rb') as f:
      return f.readlines()
  except IOError as e:
    print >> sys.stderr, e
    return None

def spew_raw_lines(file, lines):
  try:
    with open(file, 'wb') as f:
      for line in lines:
        f.write(line)
      return True
  except IOError as e:
    print >> sys.stderr, e
    return False

def update(file, lines):
  if lines is None:
    return None
  eol = "\n"
  if len(lines) and re.search(r'\r\n$', lines[0]):
    eol = "\r\n"
  result = map(lambda x: x.rstrip() + eol, lines)
  # If the trailing newline was missing, but there's no other trailing
  # whitespace, leave well enough alone.
  if len(lines) and lines[-1] == lines[-1].rstrip():
    result[-1] = lines[-1]
  if result == lines:
    return None
  return result

if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description="Remove all trailing whitespace from a file.")
  parser.add_argument('files', metavar='FILE', nargs='*',
                      help='The file names to modify.')
  args = parser.parse_args()

  touched = False
  for file in args.files:
    lines = update(file, slurp_raw_lines(file))
    if lines is not None:
      spew_raw_lines(file, lines)
      print "Updated", file
      touched = True

  if not touched:
    print "{}: Nothing to update!".format(sys.argv[0])



#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Runs AStyle to reformat the specified source files."""

import argparse
import sys
import os
import subprocess

ASTYLE_FLAGS = ('--indent=force-tab=4', '--style=allman', '--add-brackets',
                '--unpad-paren', '--pad-header', '--align-pointer=type',
                '--indent-preprocessor',
                '--min-conditional-indent=0', '--max-instatement-indent=60',
                # The following flags require AStyle 2.03 or later:
                '--max-code-length=120', '--break-after-logical',
                )

def adjust_environment():
  """Adjust the environment with some likely AStyle locations."""
  if 'SYSTEMDRIVE' in os.environ:
    for prefix in [os.environ['SYSTEMDRIVE'], 'c:', 'd:', 's:']:
      for location in 'SimQuest', 'simquest', 'tools':
        path = prefix + '/' + location + "/AStyle/bin"
        if os.path.isdir(path):
          os.environ['PATH'] += os.pathsep + path

def run_astyle(files):
  command = ['astyle']
  command.extend(ASTYLE_FLAGS)
  command.extend(files)
  try:
    retcode = subprocess.call(command)
    if retcode < 0:
      print sys.argv[0] + ": AStyle terminated by signal", -retcode
      sys.exit(1)
    elif retcode > 0:
      print sys.argv[0] + ": AStyle returned status ", retcode
      sys.exit(1)
  except OSError as e:
    print sys.argv[0] + ": AStyle execution failed:", e
    sys.exit(1)

if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description='Reformat source files using AStyle with our options.')
  parser.add_argument('files', metavar='FILE', nargs='*',
                      help='The file names to reformat.')
  args = parser.parse_args()

  adjust_environment()

  if len(args.files) == 0:
    print "{}: Nothing to reformat!".format(sys.argv[0])
  else:
    run_astyle(args.files)



#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Runs Google's cpplint.py and some other checks on C++ files.

This script tries to flag violations of the OpenSurgSim coding
standards in C++ code.

This is certainly not a substitute for looking at the code by hand:
many violations will not be caught by the script, and some amount of
flagged code will turn out to be correct.  But it does make it easier
to notice certain common errors.
"""

# TODO(bert): add some more checks.

import argparse
import sys
import subprocess
import re
import os

from fix_header_guards import get_expected_header_guard
from alphabetize_cmakelists import filename_sort_filter

STANDARD_WARNING_FORMAT = \
    "{file}:{line}:{col}: {text} [{category}] [{level}]"
VISUAL_STUDIO_WARNING_FORMAT = \
    "{file}({line},{col}): {warning} {vscategory}[{level}]: {text}"
WARNING_FORMAT = STANDARD_WARNING_FORMAT

def emit_warning(fields):
  fields = dict(fields)
  fmt = WARNING_FORMAT
  # add a "vscategory" field which is like "category" but never empty
  if 'category' in fields and fields['category'] is not None:
    fields['vscategory'] = fields['category']
  else:
    fmt = re.sub(r'\s*\[\{category[^\}]*\}\]', '', fmt)
    fmt = re.sub(r'\s*\{category[^\}]*\}', '', fmt)
    fields['category'] = '???'  # panic button
    fields['vscategory'] = 'RL'
  if 'level' not in fields:
    fmt = re.sub(r'\s*\[\{level[^\}]*\}\]', '', fmt)
    fmt = re.sub(r'\s*\{level[^\}]*\}', '', fmt)
    fields['level'] = '???'  # panic button
  if 'col' not in fields:
    fmt = re.sub(r'\s*\{col[^\}]*\}:', '', fmt)
    fmt = re.sub(r'\s*,\{col[^\}]*\}\)', ')', fmt)
    fmt = re.sub(r'\s*\{col[^\}]*\}', '', fmt)
    fields['col'] = '???'  # panic button
  if 'line' not in fields:
    fmt = re.sub(r'\s*\{line[^\}]*\}:', '', fmt)
    fmt = re.sub(r'\s*\(\{line[^\}]*\}\)', '', fmt)
    fmt = re.sub(r'\s*\{line[^\}]*\}', '', fmt)
    fields['line'] = '???'  # panic button
  if 'warning' not in fields:
    fields['warning'] = 'warning'
  print fmt.format(**fields)

def emit_error(fields):
  fields = dict(fields)
  fields['warning'] = 'error'
  emit_warning(fields)

def slurp_numbered_lines(file):
  try:
    with open(file, 'r') as f:
      lines = list(enumerate(f.read().splitlines(), start=1))
      if not lines:
        emit_warning({'file': file, 'category': "opensurgsim/no_lines",
                      'text': "file is empty."})
      return lines
  except IOError as e:
    print >> sys.stderr, e
    emit_error({'file': file, 'category': "opensurgsim/no_file",
                'text': "file is missing or could not be opened."})
    return None

def run_cpplint(script, filter, files):
  if not files:
    return False
  argv = [sys.executable, script]
  if filter is not None:
    argv.extend(['--filter', filter])
  argv.extend(files)
  try:
    cmd = subprocess.Popen(argv, stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT)
  except OSError as e:
    print "Failed to run '" + script + "':\n\t", e
    return False

  saw_output = False
  for line in cmd.stdout:
    saw_output = True
    line = line.rstrip()
    match = re.search(r'^(.*?):(\d+):\s+(.*?)\s+\[(\S+)\]\s+\[(\d+)\]\s*$',
                      line)
    if match:
      fields = dict(zip(['file', 'line', 'text', 'category', 'level'],
                        match.groups()))
      emit_warning(fields)
    elif re.search(r'\[(\S+)\]\s+\[(\d+)\]\s*$', line):
      emit_error({'file': "run-lint",
                  'text': ("failed to parse likely warning line: '" +
                           line + "'")})
    elif re.search(r':\d+:', line):
      emit_error({'file': "run-lint",
                  'text': ("failed to parse line with likely line number: '" +
                           line + "'")})
    else:
      print line

  # The output is done, so the command should be finished.
  assert cmd.wait() is not None
  if not saw_output:
    emit_error({'file': "run-lint", 'text': "cpplint produced no output!"})
    return False
  return cmd.returncode == 0

def check_header_guard(flags, file, lines):
  ifndef = filter(lambda x: re.search(r'^\s*#\s*ifndef\b', x[1]), lines)
  if not ifndef:
    emit_warning({'file': file, 'category': "opensurgsim/no_ifndef",
                  'text': "no #ifndef lines!"})
    return False
#  elif len(ifndef) > 1:
#    emit_warning({'file': file, 'line': ifndef[1][0],
#                  'category': "opensurgsim/several_ifndef",
#                  'text': "multiple #ifndef lines; using first."})

  guard = ifndef[0][1]
  guard = re.sub(r'//.*', '', guard)
  guard = re.sub(r'/\*.*?\*/', '', guard)
  guard_match = re.search(r'^\s*#\s*ifndef\s+(\w+)\s*$', guard)
  if not guard_match:
    emit_error({'file': file, 'line': ifndef[0][0],
                'category': "opensurgsim/internal",
                'text': "script error: failed to parse #ifndef line!"})
    return False
  guard = guard_match.group(1)

  guard_expected = get_expected_header_guard(file)

  if guard != guard_expected:
    emit_warning({'file': file, 'line': ifndef[0][0],
                  'category': "opensurgsim/header_guard",
                  'text': ("unexpected guard '{}'!  expected '{}'."
                           .format(guard, guard_expected)) })

  define = filter(lambda x: re.search(r'^\s*#\s*define\b', x[1]), lines)
  if not define:
    emit_warning({'file': file, 'category': "opensurgsim/no_define",
                  'text': "no #define lines!"})
  else:
    def_match = re.match(r'^\s*#\s*define\s+(\w+)\s*$',
                         re.sub(r'/\*.*?\*/', '',
                                re.sub(r'//.*', '', define[0][1])))
    if not def_match:
      emit_error({'file': file, 'line': define[0][0],
                  'category': "opensurgsim/internal",
                  'text': "script error: failed to parse #define line!"})
    elif def_match.group(1) != guard:
      emit_warning({'file': file, 'line': define[0][0],
                    'category': "opensurgsim/guard_mismatch",
                    'text': "#define doesn't match #ifndef!"})

  endif = filter(lambda x: re.search(r'^\s*#\s*endif\b', x[1]), lines)
  if not endif:
    emit_warning({'file': file, 'category': "opensurgsim/no_endif",
                  'text': "no #endif lines!"})
  elif re.search(r'^\s*#\s*endif\s*$', endif[-1][1]):
    if flags.missing_endif_comments:
      emit_warning({'file': file, 'line': endif[-1][0],
                    'category': "opensurgsim/endif_no_comment",
                    'text': "#endif with no comment."})
  else:
    if re.search(r'^\s*#\s*endif\s*///', endif[-1][1]):
      emit_warning({'file': file, 'line': endif[-1][0],
                    'category': "opensurgsim/endif_doxygen",
                    'text': "#endif with a Doxygen style /// comment."})
    endif_match = re.search(r'^\s*#\s*endif\s*//+\s*(.*?)\s*$', endif[-1][1])
    if not endif_match:
      emit_error({'file': file, 'line': endif[-1][0],
                  'category': "opensurgsim/internal",
                  'text': "script error: failed to parse #endif comment."})
    elif endif_match.group(1) != guard:
      emit_warning({'file': file, 'line': endif[-1][0],
                    'category': "opensurgsim/endif_mismatch",
                    'text': ("#endif comment doesn't match #ifndef! (" + \
                               endif_match.group(1) + ")")})

def find_column_char(text, column, tab_width=4):
  """Find the character that corresponds to the specified column.

  In the absence of tabs in the text, the return value will be equal
  to the column.  When tabs are present, it will be smaller.

  """
  text = text[:column]
  while len(text.expandtabs(tab_width)) > column:
    text = text[:(len(text)-1)]
  return len(text)

def get_column(flags, text, column):
  if flags.visual_studio:
    return find_column_char(text, column)
  else:
    return column

def check_length(flags, file, lines):
  xlines = map(lambda x: (x[0], re.sub(r'\r?\n$', '', x[1].expandtabs(4))),
               lines)
  for bad in filter(lambda x: len(x[1]) > flags.max_line_length, xlines):
    col = get_column(flags, lines[bad[0]-1][1], flags.max_line_length)
    emit_warning({'file': file, 'line': bad[0], 'col': col,
                  'category': "opensurgsim/too_long",
                  'text': ("the line is {} characters long, which is longer"
                           " than {}!"
                           .format(len(bad[1]), flags.max_line_length)) })

def check_tab_indentation(flags, file, lines):
  for line_number, line in lines:
    # Not a blank line
    if len(line) > 0:
      # Check that lines start with 0 or more tab characters.
      # 0 to 3 spaces is allowed for alignment, but anymore and a tab is expected.
      if re.search(r'^\t* {0,3}\S' , line) is None:
        emit_warning({'file': file, 'line': line_number,
                      'category': "opensurgsim/tab_indention",
                      'text': "the line does not correctly indent using tabs."})

def check_oss_include(flags, file, lines):
  for line_number, line in lines:
    # Not a blank line
    if len(line) > 0:
      # Check that lines does not start with "#include <SurgSim"
      if re.search(r'#include <SurgSim' , line) is not None:
        emit_warning({'file': file, 'line': line_number,
                      'category': "opensurgsim/oss_include",
                      'text': "OSS header files should be included using #include \"\"."})

def get_listed_files(flags, file, lines):
  flines = filter(lambda x: re.search(r'^\s.*\.(?:h|cpp)\s*(?:\)\s*)?$', x[1]),
                  lines)
  for f in flines:
    if not re.search(r'^\t\S', f[1]):
      emit_warning({'file': file, 'line': f[0],
                    'category': "opensurgsim/cmake_file_indent",
                    'text': "file name indentation is not a single tab."})
    if re.search(r'\)\s*$', f[1]):
      emit_warning({'file': file, 'line': f[0],
                    'category': "opensurgsim/cmake_file_parenthesis",
                    'text': "file name is followed by a right parenthesis."})
    elif re.search(r'\s$', f[1]):
      emit_warning({'file': file, 'line': f[0],
                    'category': "opensurgsim/cmake_file_trailing",
                    'text': "file name is followed by trailing whitespace."})
  files = map(lambda x: (x[0], re.sub(r'\s*\)$', '', x[1].strip())), flines)
  for f in files:
    if re.search(r'\$', f[1]):
      emit_warning({'file': file, 'line': f[0],
                    'category': "opensurgsim/cmake_file_variable",
                    'text': "file name contains a variable!"})
    if re.search(r'\s', f[1]):
      emit_warning({'file': file, 'line': f[0],
                    'category': "opensurgsim/cmake_file_whitespace",
                    'text': "file name contains whitespace!"})
  return files

def check_file_order(flags, file, files):
  saw_duplicated = False
  saw_out_of_order = False
  for i in range(1, len(files)):
    if files[i-1][0] == (files[i][0]-1):
      first  = filename_sort_filter(files[i-1][1])
      second = filename_sort_filter(files[i][1])
      if first == second and not saw_duplicated:
        emit_warning({'file': file, 'line': files[i][0],
                      'category': "opensurgsim/cmake_file_duplicate",
                      'text': "file list contains duplicate entries." })
        saw_duplicated = False
      elif first > second and not saw_out_of_order:
        emit_warning({'file': file, 'line': files[i][0],
                      'category': "opensurgsim/cmake_file_order",
                      'text': "file list is not correctly ordered." })
        saw_out_of_order = False

def decorate_listed_files(flags, file, files):
  prefix = os.path.dirname(file)
  paths = map(lambda x: (x[0], os.path.normpath(os.path.join(prefix, x[1]))),
              files)
  for p in paths:
    if not os.path.exists(p[1]):
      emit_warning({'file': file, 'line': p[0],
                    'category': "opensurgsim/cmake_file_missing",
                    'text': ("file " + p[1] +
                             " does not exist but is listed in CMakeLists!") })
  return paths

def get_listed_file_paths(flags, file, lines):
  files = get_listed_files(flags, file, lines)
  check_file_order(flags, file, files)
  paths = decorate_listed_files(flags, file, files)
  return map(lambda x: x[1], paths)

def find_responsible_cmakelists(file):
  dir_path = os.path.dirname(file)
  while True:
    guess = os.path.join(dir_path, 'CMakeLists.txt')
    if os.path.exists(guess):
      return guess
    parent = os.path.dirname(dir_path)
    if parent == dir_path:
      # uh-oh, we didn't find ANY CMakeLists.txt file?!?
      return file
    dir_path = parent

def check_file_lists(cmakelists_files, found_files):
  while cmakelists_files or found_files:
    if (cmakelists_files and found_files and
        cmakelists_files[0] == found_files[0]):
      # lists match.  remove matching elements and keep going.
      del cmakelists_files[0]
      del found_files[0]
    elif not found_files or cmakelists_files[0] < found_files[0]:
      # in CMakeLists but not on disk.  should already have been handled.
      del cmakelists_files[0]
    elif not cmakelists_files or found_files[0] < cmakelists_files[0]:
      # in the list but not in CMakeLists.
      emit_warning({'file': find_responsible_cmakelists(found_files[0]),
                    'category': "opensurgsim/cmake_file_not_listed",
                    'text': ("file " + found_files[0] +
                             " not present in CMakeLists.txt!") })
      del found_files[0]
    else:
      assert False, "should not have reached here"


if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description='Check source files for coding standard violations.')
  parser.add_argument('--traverse', metavar='DIR', action='append',
                      help='Find all applicable files in DIR and its subdirs.')
  parser.add_argument('--cpplint-script', metavar='PATH_TO_SCRIPT',
                      help='The path to Google\'s cpplint.py script.',
                      default='cpplint.py')
  parser.add_argument('--cpplint-filter', '--filter', metavar='FILTER',
                      help='The --filter argument to pass to cpplint.')
  parser.add_argument('--no-cpplint', action='store_false', dest='do_cpplint',
                      help='Do not run the cpplint.py script.')
  parser.add_argument('--missing-endif-comments', action='store_true',
                      help='Flag missing include guard #endif comments, too.')
  parser.add_argument('--ignore-guards',
                      action='store_false', dest='do_check_guards',
                      help='Do not check the header file include guards.')
  parser.add_argument('--ignore-tab-indentation',
                      action='store_false', dest='do_check_tab_indentation',
                      help='Do not check that lines are indented with tabs.')
  parser.add_argument('--ignore-oss-include',
                      action='store_false', dest='do_check_oss_include',
                      help='Do not check that OSS related include use #include "".')
  parser.add_argument('--max-line-length', type=int, default=120,
                      help='Maximum allowed line length for C++ code.')
  parser.add_argument('--ignore-length',
                      action='store_false', dest='do_check_length',
                      help='Do not check the line length.')
  parser.add_argument('--ignore-file-lists',
                      action='store_false', dest='do_check_file_lists',
                      help='Do not check the files in CMakeLists.txt.')
  parser.add_argument('--ignore-ext',
                      action='store_false', dest='do_check_extension',
                      help='Do not check the file extensions.')
  parser.add_argument('--format',
                      help='The format string for warnings and errors.')
  parser.add_argument('--visual-studio', '--vs', action='store_true',
                      help='Use defaults tailored for Visual Studio.')
  parser.add_argument('files', metavar='FILE', nargs='*',
                      help='The file names to check.')
  args = parser.parse_args()

  if args.format is not None:
    WARNING_FORMAT = args.format
  elif args.visual_studio:
    WARNING_FORMAT = VISUAL_STUDIO_WARNING_FORMAT
  else:
    WARNING_FORMAT = STANDARD_WARNING_FORMAT

  if args.traverse is not None:
    for traverse_dir in args.traverse:
      for current_dir, subdirs, files in os.walk(traverse_dir):
        args.files.extend(map(lambda x: os.path.join(current_dir, x),
                              filter(lambda x: re.search(r'\.(?:h|cpp)$', x),
                                     files)))
        if 'CMakeLists.txt' in files:
          args.files.append(os.path.join(current_dir, 'CMakeLists.txt'))
        # always skip .git, build directories, ThirdParty...
        for bad in filter(lambda x:
                            re.search(r'^(?:\.git|build.*|ThirdParty|Modules)$', x, re.IGNORECASE),
                          subdirs):
          subdirs.remove(bad)

  ok = True

  if args.do_check_extension:
    bad_ext = filter(lambda x:
                       not re.search(r'\.(?:h|cpp)$|CMakeLists\.txt$', x),
                     args.files)
    for file in bad_ext:
      emit_warning({'file': file,
                    'text': "unknown extension (expected .h or .cpp)"})
      ok = False

  if args.do_cpplint:
    # Batch the files through cpplint, 100 at a time so we don't exceed OS limits on
    # command size or amount of parameters
    index = 0
    ok = True
    while index < len(args.files) :    
        if not run_cpplint(args.cpplint_script, args.cpplint_filter,
                       filter(lambda x: re.search(r'\.(?:h|cpp)$', x),
                              args.files[index:index+100])):
            ok = False
        index += 100

  cmakelists_files = None
  for file in args.files:
    lines = None

    if args.do_check_guards and re.search(r'\.h$', file):
      if lines is None:
        lines = slurp_numbered_lines(file)
        if not lines:
          continue
      if not check_header_guard(args, file, lines):
        ok = False

    if args.do_check_length and re.search(r'\.(h|cpp)$', file):
      if lines is None:
        lines = slurp_numbered_lines(file)
        if not lines:
          continue
      if not check_length(args, file, lines):
        ok = False

    if args.do_check_file_lists and re.search(r'CMakeLists\.txt$', file):
      if lines is None:
        lines = slurp_numbered_lines(file)
        if not lines:
          continue
        if cmakelists_files is None:
            cmakelists_files = []
        cmakelists_files.extend(get_listed_file_paths(args, file, lines))
        # cmakelists_files will be checked later...

    if args.do_check_tab_indentation and re.search(r'(\.h|\.cpp|CMakeLists\.txt$)', file):
      if lines is None:
        lines = slurp_numbered_lines(file)
        if not lines:
          continue
      if not check_tab_indentation(args, file, lines):
        ok = False

    if args.do_check_oss_include and re.search(r'(\.h|\.cpp$)', file):
      if lines is None:
        lines = slurp_numbered_lines(file)
        if not lines:
          continue
      if not check_oss_include(args, file, lines):
        ok = False

  if args.do_check_file_lists and cmakelists_files is not None:
    cmakelists_files = sorted(cmakelists_files)
    found_files = sorted(map(lambda x: os.path.normpath(x),
                              filter(lambda x: re.search(r'\.(?:h|cpp)$', x),
                                     args.files)))
    check_file_lists(cmakelists_files, found_files)


#  if not ok:
#    sys.exit(1)



#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Fix the header include guards in files."""

import argparse
import sys
import subprocess
import re
import os

def slurp_numbered_lines(file):
  try:
    with open(file, 'r') as f:
      lines = list(enumerate(f.read().splitlines(), start=0))
      if not lines:
        print >> sys.stderr, file + ": warning, file is empty."
      return lines
  except IOError as e:
    print >> sys.stderr, e
    return None

def spew_raw_lines(file, lines):
  try:
    with open(file, 'wb') as f:
      for line in lines:
        f.write(line)
      return True
  except IOError as e:
    print >> sys.stderr, e
    return False

HEADER_GUARD_DIRECTORY_PREFIX_CACHE = {}

def get_header_guard_directory_prefix(dir_path):
  if dir_path not in HEADER_GUARD_DIRECTORY_PREFIX_CACHE:
    prefix = ""
    path = dir_path
    while not (os.path.exists(os.path.join(path, 'LICENSE')) or
               os.path.exists(os.path.join(path, '.git'))):
      (parent, dirname) = os.path.split(path)
      assert parent != path
      prefix = re.sub(r'\W+', '', dirname).upper() + '_' + prefix
      path = parent
    HEADER_GUARD_DIRECTORY_PREFIX_CACHE[dir_path] = prefix

  return HEADER_GUARD_DIRECTORY_PREFIX_CACHE[dir_path]

def get_expected_header_guard(file):
  (dir_path, filename) = os.path.split(os.path.abspath(file))
  guard = re.sub(r'\W+', '_', filename).upper()

  while not (os.path.exists(os.path.join(dir_path, 'LICENSE')) or
             os.path.exists(os.path.join(dir_path, '.git'))):
    (parent, dirname) = os.path.split(dir_path)
    assert parent != dir_path
    guard = re.sub(r'\W+', '', dirname).upper() + '_' + guard
    dir_path = parent

  return guard

def fix_header_guards(flags, file, lines):
  ifndef = filter(lambda x: re.search(r'^\s*#\s*ifndef\b', x[1]), lines)
  if not ifndef:
    print >> sys.stderr, file + ": no #ifndef lines!"
    return None

  guard_expected = get_expected_header_guard(file)

  lines[ifndef[0][0]] = (ifndef[0][0],
                         re.sub(r'^(\s*#\s*ifndef)\s+\w+((?:\s.*)?)$',
                                r'\1 ' + guard_expected + r'\2',
                                ifndef[0][1]))

  define = filter(lambda x: re.search(r'^\s*#\s*define\b', x[1]), lines)
  if not define:
    print >> sys.stderr, file + ": no #define lines!"
    return None

  lines[define[0][0]] = (define[0][0],
                         re.sub(r'^(\s*#\s*define)\s+\w+((?:\s.*)?)$',
                                r'\1 ' + guard_expected + r'\2',
                                define[0][1]))

  endif = filter(lambda x: re.search(r'^\s*#\s*endif\b', x[1]), lines)
  if not endif:
    print >> sys.stderr, file + ": no #endif lines!"
    return None

  update_endif = False
  if not args.lenient_endif:
    update_endif = True
  if (re.search(r'^\s*#\s*endif\s*//', endif[-1][1]) and
      not re.search(r'^\s*#\s*endif\s+//\s+' + guard_expected + r'\s*$',
                    endif[-1][1])):
    update_endif = True  # The comment exists but isn't right.

  if update_endif:
    lines[endif[-1][0]] = (endif[-1][0],
                           re.sub(r'^(\s*#\s*endif)\b.*$',
                                  r'\1  // ' + guard_expected,
                                  endif[-1][1]))
  return lines

if __name__ == '__main__':
  parser = argparse.ArgumentParser(
    description='Fix the header include guards in files.')
  parser.add_argument('--lenient-endif', action='store_true',
                      help='Fix #endif comments only if present and wrong.')
  parser.add_argument('files', metavar='FILE', nargs='*',
                      help='The file names to check.')
  args = parser.parse_args()

  for file in args.files:
    if not re.search(r'\.h$', file):
      print >> sys.stderr, file + ": Ignored."
      continue

    lines = slurp_numbered_lines(file)
    if not lines:
      continue
    updated = fix_header_guards(args, file,
                                map(lambda x: (x[0], x[1]), lines))
    if not updated:
      continue
    if updated == lines:
      continue
    output = map(lambda x: x[1] + "\n", updated)
    spew_raw_lines(file, output)
    print >> sys.stderr, file + ": Updated."



#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2013, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Merge a list of input vertices and information from an .obj file

Give a file with a list of input vertices, look through the obj file and find
corresponding vertices, add texture information to the input vertices and store
the resulting list in an output file. The order in the vertex file is maintained
and vertices that cannot be found in the obj file will receive default information
This can be used to add information to a .ply file where the order of the vertices
is already established
E.g. 

vertex file (fragment from .ply file)
1.0 1.0 1.0
2.0 2.0 2.0 
3.0 3.0 3.0

obj file (fragment from .obj file)
v 1.0 1.0 -1.0
v 2.0 2.0 -2.0
vt 1.0 1.0
vt 0.5 0.5

output (fragment, paste this into the vertex section of the .ply file)
1.0 1.0 1.0 1.0 1.0
2.0 2.0 2.0 0.5 0.5
3.0 3.0 3.0 0.0 0.0

Typical usage:
  merge.py input.txt object.obj output.txt
"""

import csv
import argparse

def find(item, vertices) :
	index = 0
	for vertex in vertices:
		if (item == vertex) :
			return index
		index += 1
	return -1
	
vertices = []
uvs = []
uvIndices = []


if __name__ == '__main__':
	parser = argparse.ArgumentParser(
	description="Merge a list of input vertices and information from an .obj file.")
	parser.add_argument('vertices', help='Filename for input vertices.')
	parser.add_argument('obj', help='The .obj file to use.' )
	parser.add_argument('output', help='Filename for output.')
	args = parser.parse_args()

	# this can be a basic obj file, we are only looking at 'v' and 'vt' for now
	with open(args.obj, 'rb') as csvfile:
		reader = csv.reader(csvfile, delimiter = ' ')
		
		vertexIndex = 0
		for row in reader: 
			if (row[0] == 'v') :
				#Go from obj to ply coordinate system, change the sign on the z coord
				if (row[3][0] == '-'):
					row[3] = row[3][1:]
				else:
					row[3] = '-'+row[3]
					
				#swap y and z
				tmp = row[3]
				row[3] = row[2]
				row[2] = tmp			
				vertices.append(row[1:])
			elif (row[0] == 'vt') :
				uvs.append(row[1:])
				uvIndices.append(vertexIndex)
				vertexIndex += 1
			elif (row[0] == 'f') :
				# These are either 
				# f v v v
				# f v/vt v/vt v/vt
				# f v/vt/vn v/vt/vn v/vt/vn
				for item in row[1:]:
					indices = item.split("/")
					if (len(indices) > 1) :
						# .obj indices start with 1
						# put the index of the texture coordinate into the indices map
						uvIndices[int(indices[0])-1] = int(indices[1])-1
	
	result = []
	 
	# Reference is just a list of vertices x,y,z separated by ' '
	# The comparison is done via the strings of the numbers, no accuracy needed, this relies
	# on the fact that the output format is the same and no transformation was done on the
	# vertices
	with open(args.vertices, 'rb') as csvfile:
		reader = csv.reader(csvfile, delimiter = ' ')
		count = 0
		for row in reader:
			texCoord = ["0.0","0.0"]
			index = find(row, vertices)
			if (index != -1) :
				texCoord = uvs[uvIndices[index]]
			row.append(texCoord[0])
			row.append(texCoord[1])
			result.append(row)

			
	with open(args.output, 'wb') as outfile:
		writer = csv.writer(outfile, delimiter = ' ', quoting=csv.QUOTE_NONE)
		for row in result:
			writer.writerow(row)


#!/usr/bin/python

# This file is a part of the OpenSurgSim project.
# Copyright 2012-2015, SimQuest Solutions Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Convert a TetGen set of files .node/.ele into a PLY file readable by OSS

Typical usage:
  TetGen_to_PLY.py input.node input.ele [input.face] [input.fixedNodes] output.ply
"""

import csv
import argparse

if __name__ == '__main__':
	parser = argparse.ArgumentParser(
	description="Convert a TetGen set of filename into a PLY file readable by OSS.")
	parser.add_argument('nodes', help='Filename for the nodes input.')
	parser.add_argument('elements', help='Filename for the tetrahedrons input.')
	parser.add_argument('--faces', help='Filename for the faces input.')
	parser.add_argument('--fixedNodes', help='Filename for the fixed node indices.')
	parser.add_argument('--massDensity', help='Mass density.', default='2000.0')
	parser.add_argument('--youngModulus', help='Young modulus.', default='1e7')
	parser.add_argument('--poissonRatio', help='Poisson ratio.', default='0.45')
	parser.add_argument('output', help='Filename for the PLY output.')
	args = parser.parse_args()

	numNodes = 0        # Number of nodes, will be read from the header of the .node file
	numElements = 0     # Number of elements, will be read from the header of the .ele file (support triangle (3) and tetrahedron(4) elements)
	elementSize = 0     # Element size 3 for triangle, 4 for tetrahedron, will be read from the header of the .ele file
	numFaces = 0        # Number of triangulated faces (if a .face is provided, number of entries in this file)
	numFixedNodes = 0   # Number of fixed nodes (if a .fixedNode is provided, number of entries in this file)

	with open(args.output, 'wb') as csvOutputFile:
		writer = csv.writer(csvOutputFile, delimiter = ' ', quoting=csv.QUOTE_NONE)

		writer.writerow(['ply'])
		writer.writerow(['format', 'ascii', '1.0'])
		writer.writerow(['comment', 'Created', 'by', 'tetgen2ply'])

		with open(args.nodes, 'rb') as csvNodeFile:
			reader = csv.reader(csvNodeFile, delimiter = ' ', skipinitialspace = True)
			row = reader.next()
			numNodes = row[0]
			# Enforcing the need for 3 pieces of information per node (x, y, z)
			if not row[1] == '3':
				raise Exception('Invalid node information in ' + args.node + '. Node dimension (expecting 3) was ' + row[1])
		writer.writerow(['element', 'vertex', numNodes])
		writer.writerow(['property', 'double', 'x'])
		writer.writerow(['property', 'double', 'y'])
		writer.writerow(['property', 'double', 'z'])

		with open(args.elements, 'rb') as csvElementFile:
			reader = csv.reader(csvElementFile, delimiter = ' ', skipinitialspace = True)
			row = reader.next()
			numElements = row[0]
		if row[1] == '4':
			writer.writerow(['element', '3d_element', numElements])
			elementSize = 4
		elif row[1] == '3':
			writer.writerow(['element', '2d_element', numElements])
			elementSize = 3
		else :
			raise Exception('Invalid triangle/tetrahedron information in ' + args.ele + '. Element dimension (expecting 3 or 4) was ' + row[1])
		writer.writerow(['property', 'list', 'uint', 'uint', 'vertex_indices'])

		if args.faces:
			with open(args.faces, 'rb') as csvFaceFile:
				reader = csv.reader(csvFaceFile, delimiter = ' ', skipinitialspace = True)
				row = reader.next()
				numFaces = row[0]
			writer.writerow(['element', 'face', numFaces])
			writer.writerow(['property', 'list', 'uint', 'uint', 'vertex_indices'])

		if args.fixedNodes:
			with open(args.fixedNodes, 'rb') as csvFixedNodeFile:
				readerFixedNodes = csv.reader(csvFixedNodeFile)
				for row in readerFixedNodes:
					numFixedNodes = numFixedNodes + 1
			writer.writerow(['element', 'boundary_condition', numFixedNodes])
			writer.writerow(['property', 'uint', 'vertex_index'])

		# Extra parameter (thickness) if the element is a triangle
		if elementSize == 3:
			writer.writerow(['element', 'thickness', 1])
			writer.writerow(['property', 'double', 'value'])

		writer.writerow(['element', 'material', 1])
		writer.writerow(['property', 'double', 'mass_density'])
		writer.writerow(['property', 'double', 'poisson_ratio'])
		writer.writerow(['property', 'double', 'young_modulus'])
		writer.writerow(['end_header'])

		# Parse the .node file to format the nodes (x,y,z)
		with open(args.nodes, 'rb') as csvNodeFile:
			reader = csv.reader(csvNodeFile, delimiter = ' ', skipinitialspace = True)
			rowId = 0
			# Write all nodes
			for row in reader:
				# Skip the commented lines (especially the last line of the .node generated by TetGen)
				if row[0][0] == '#':
					continue
				# Skip the first line (header information), detected by the number of nodes being different than the expected node index
				if not int(row[0]) == rowId:
					continue
				writer.writerow(row[1:])
				rowId = rowId + 1

		# Parse the .ele file to format the tetrahedrons
		with open(args.elements, 'rb') as csvElementFile:
			reader = csv.reader(csvElementFile, delimiter = ' ', skipinitialspace = True)
			rowId = 0
			# Write all tetrahedrons
			for row in reader:
				# Skip the commented lines (especially the last line of the .ele generated by TetGen)
				if row[0][0] == '#':
					continue
				# Skip the first line (header information), detected by the number of elements being different than the expected element index
				if not int(row[0]) == rowId:
					continue
				row[0] = elementSize
				writer.writerow(row)
				rowId = rowId + 1

		# Parse the .face file to format the triangulated faces (if a file is specified)
		if args.faces:
			with open(args.faces, 'rb') as csvFaceFile:
				reader = csv.reader(csvFaceFile, delimiter = ' ', skipinitialspace = True)
				rowId = 0
				# Write all faces
				for row in reader:
					# Skip the commented lines (especially the last line of the .face generated by TetGen)
					if row[0][0] == '#':
						continue
					# Skip the first line (header information), detected by the number of faces being different than the expected face index
					if not int(row[0]) == rowId:
						continue
					row[0] = 3
					writer.writerow(row[:4])
					rowId = rowId + 1

		# Write the fixed nodes (boundary conditions) if any
		if args.fixedNodes:
			with open(args.fixedNodes, 'rb') as csvFixedNodeFile:
				readerFixedNodes = csv.reader(csvFixedNodeFile, delimiter = ' ', skipinitialspace = True)
				for row in readerFixedNodes:
					writer.writerow(row)

		# Write a default thickness if the element is a triangle
		# We should have something closer to 1/100th of the mesh size if we wanted to have a more automated tool
		if elementSize == 3:
			writer.writerow(['0.01'])

		# Write the material (default parameters)
		writer.writerow([args.massDensity, args.poissonRatio, args.youngModulus])
